@@ -1,9 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
-import bisect
 from collections import Iterator
 from datetime import datetime
-from distutils.version import LooseVersion
 import math
 import operator
 from operator import getitem, setitem
@@ -27,9 +25,12 @@ from ..array.core import partial_by_order
 from .. import threaded
 from ..compatibility import apply, operator_div, bind_method
 from ..utils import (repr_long_list, IndexCallable,
-                     pseudorandom, derived_from, different_seeds)
+                     pseudorandom, derived_from, different_seeds, funcname)
 from ..base import Base, compute, tokenize, normalize_token
 from ..async import get_sync
+from .indexing import (_partition_of_index_value, _loc, _try_loc,
+                       _coerce_loc_index, _maybe_partial_time_string)
+from .utils import nonempty_sample_df
 
 no_default = '__no_default__'
 return_scalar = '__return_scalar__'
@@ -43,8 +44,6 @@ def _concat(args, **kwargs):
         return args
     if isinstance(first(core.flatten(args)), np.ndarray):
         return da.core.concatenate3(args)
-    if len(args) == 1:
-        return args[0]
     if isinstance(args[0], (pd.DataFrame, pd.Series)):
         args2 = [arg for arg in args if len(arg)]
         if not args2:
@@ -73,7 +72,7 @@ class Scalar(Base):
 
     _optimize = staticmethod(optimize)
     _default_get = staticmethod(threaded.get)
-    _finalize = staticmethod(finalize)
+    _finalize = staticmethod(first)
 
     def __init__(self, dsk, _name, name=None, divisions=None):
         self.dask = dsk
@@ -102,7 +101,7 @@ class Scalar(Base):
     @classmethod
     def _get_unary_operator(cls, op):
         def f(self):
-            name = tokenize(self)
+            name = funcname(op) + '-' + tokenize(self)
             dsk = {(name, 0): (op, (self._name, 0))}
             return Scalar(merge(dsk, self.dask), name)
         return f
@@ -113,7 +112,7 @@ class Scalar(Base):
 
 
 def _scalar_binary(op, a, b, inv=False):
-    name = '{0}-{1}'.format(op.__name__, tokenize(a, b))
+    name = '{0}-{1}'.format(funcname(op), tokenize(a, b))
 
     dsk = a.dask
     if not isinstance(b, Base):
@@ -162,7 +161,7 @@ class _Frame(Base):
 
     def __new__(cls, dsk, _name, metadata, divisions):
         if (np.isscalar(metadata) or metadata is None or
-            isinstance(metadata, (Series, pd.Series))):
+           isinstance(metadata, (Series, pd.Series))):
             return Series(dsk, _name, metadata, divisions)
         else:
             return DataFrame(dsk, _name, metadata, divisions)
@@ -201,9 +200,13 @@ class _Frame(Base):
         else:
             if np.isscalar(metadata) or metadata is None:
                 _pd = cls._partition_type([], name=metadata)
+                known_dtype = False
+            elif isinstance(metadata, datetime):
+                _pd = cls._partition_type([metadata]).iloc[0:0]
+                known_dtype = True
             else:
                 _pd = cls._partition_type(columns=metadata)
-            known_dtype = False
+                known_dtype = False
         return _pd, known_dtype
 
     @property
@@ -392,12 +395,13 @@ class _Frame(Base):
         if isinstance(ind, Series):
             return self._loc_series(ind)
         if self.known_divisions:
+            ind = _maybe_partial_time_string(self._pd.index, ind, kind='loc')
             if isinstance(ind, slice):
                 return self._loc_slice(ind)
             else:
                 return self._loc_element(ind)
         else:
-            return map_partitions(try_loc, self, self, ind)
+            return map_partitions(_try_loc, self, self, ind)
 
     def _loc_series(self, ind):
         if not self.divisions == ind.divisions:
@@ -406,7 +410,7 @@ class _Frame(Base):
                               self._pd, self, ind, token='loc-series')
 
     def _loc_element(self, ind):
-        name = 'loc-element-%s-%s' % (str(ind), self._name)
+        name = 'loc-%s' % tokenize(ind, self)
         part = _partition_of_index_value(self.divisions, ind)
         if ind < self.divisions[0] or ind > self.divisions[-1]:
             raise KeyError('the label [%s] is not in the index' % str(ind))
@@ -415,8 +419,9 @@ class _Frame(Base):
         return self._constructor(merge(self.dask, dsk), name, self, [ind, ind])
 
     def _loc_slice(self, ind):
-        name = 'loc-slice-%s-%s' % (str(ind), self._name)
+        name = 'loc-%s' % tokenize(ind, self)
         assert ind.step in (None, 1)
+
         if ind.start:
             start = _partition_of_index_value(self.divisions, ind.start)
         else:
@@ -457,13 +462,8 @@ class _Frame(Base):
         >>> df.loc["b":"d"]  # doctest: +SKIP"""
         return IndexCallable(self._loc)
 
-    @property
-    def iloc(self):
-        """ Not implemented """
-
-        # not implemented because of performance concerns.
-        # see https://github.com/dask/dask/pull/507
-        raise NotImplementedError("Dask Dataframe does not support iloc")
+    # NOTE: `iloc` is not implemented because of performance concerns.
+    # see https://github.com/dask/dask/pull/507
 
     def repartition(self, divisions=None, npartitions=None, force=False):
         """ Repartition dataframe along new divisions
@@ -542,7 +542,7 @@ class _Frame(Base):
 
     @derived_from(pd.DataFrame)
     def to_hdf(self, path_or_buf, key, mode='a', append=False, complevel=0,
-               complib=None, fletcher32=False, get=get_sync, **kwargs):
+               complib=None, fletcher32=False, get=None, **kwargs):
         from .io import to_hdf
         return to_hdf(self, path_or_buf, key, mode, append, complevel, complib,
                 fletcher32, get=get, **kwargs)
@@ -561,8 +561,8 @@ class _Frame(Base):
 
         Returns a list of values, one value per partition.
         """
-        from ..delayed import Value
-        return [Value(k, [self.dask]) for k in self._keys()]
+        from ..delayed import Delayed
+        return [Delayed(k, [self.dask]) for k in self._keys()]
 
     @classmethod
     def _get_unary_operator(cls, op):
@@ -575,58 +575,105 @@ class _Frame(Base):
         else:
             return lambda self, other: elemwise(op, self, other)
 
-    def _aca_agg(self, token, func, aggfunc=None):
+    def _aca_agg(self, token, func, aggfunc=None, **kwargs):
         """ Wrapper for aggregations """
         raise NotImplementedError
 
+    def rolling(self, window, min_periods=None, freq=None, center=False,
+                win_type=None, axis=0):
+        """Provides rolling transformations.
+
+        Parameters
+        ----------
+        window : int
+           Size of the moving window. This is the number of observations used
+           for calculating the statistic. The window size must not be so large
+           as to span more than one adjacent partition.
+        min_periods : int, default None
+            Minimum number of observations in window required to have a value
+            (otherwise result is NA).
+        center : boolean, default False
+            Set the labels at the center of the window.
+        win_type : string, default None
+            Provide a window type. The recognized window types are identical
+            to pandas.
+        axis : int, default 0
+
+        Returns
+        -------
+        a Rolling object on which to call a method to compute a statistic
+
+        Notes
+        -----
+        The `freq` argument is not supported.
+        """
+        from dask.dataframe.rolling import Rolling
+
+        if not isinstance(window, int):
+            raise ValueError('window must be an integer')
+        if window < 0:
+            raise ValueError('window must be >= 0')
+
+        if min_periods is not None:
+            if not isinstance(min_periods, int):
+                raise ValueError('min_periods must be an integer')
+            if min_periods < 0:
+                raise ValueError('min_periods must be >= 0')
+
+        return Rolling(self, window=window, min_periods=min_periods,
+                       freq=freq, center=center, win_type=win_type, axis=axis)
+
     @derived_from(pd.DataFrame)
     def sum(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.sum(axis=1, skipna=skipna)
-            name = '{0}sum(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_sum, None, self,
+                                  token=self._token_prefix + 'sum',
+                                  axis=axis, skipna=skipna)
         else:
-            return self._aca_agg(token='sum', func=lambda x: x.sum(skipna=skipna))
+            return self._aca_agg(token='sum', func=_sum,
+                                 skipna=skipna, axis=axis)
 
     @derived_from(pd.DataFrame)
     def max(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.max(axis=1, skipna=skipna)
-            name = '{0}max(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_max, None, self,
+                                  token=self._token_prefix + 'max',
+                                  skipna=skipna, axis=axis)
         else:
-            return self._aca_agg(token='max', func=lambda x: x.max(skipna=skipna))
+            return self._aca_agg(token='max', func=_max,
+                                 skipna=skipna, axis=axis)
 
     @derived_from(pd.DataFrame)
     def min(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.min(axis=1, skipna=skipna)
-            name = '{0}min(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_min, None, self,
+                                  token=self._token_prefix + 'min',
+                                  skipna=skipna, axis=axis)
         else:
-            return self._aca_agg(token='min', func=lambda x: x.min(skipna=skipna))
+            return self._aca_agg(token='min', func=_min,
+                                 skipna=skipna, axis=axis)
 
     @derived_from(pd.DataFrame)
     def count(self, axis=None):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.count(axis=1)
-            name = '{0}count(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_count, None, self,
+                                  token=self._token_prefix + 'count',
+                                  axis=axis)
         else:
-            return self._aca_agg(token='count', func=lambda x: x.count(),
+            return self._aca_agg(token='count', func=_count,
                                  aggfunc=lambda x: x.sum())
 
     @derived_from(pd.DataFrame)
     def mean(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.mean(axis=1, skipna=skipna)
-            name = '{0}mean(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_mean, None, self,
+                                  token=self._token_prefix + 'mean',
+                                  axis=axis, skipna=skipna)
         else:
             num = self._get_numeric_data()
             s = num.sum(skipna=skipna)
@@ -637,16 +684,16 @@ class _Frame(Base):
                     return s / n
                 except ZeroDivisionError:
                     return np.nan
-            name = '{0}mean-{1}'.format(self._token_prefix, tokenize(s))
+            name = self._token_prefix + 'mean-%s' % tokenize(self, axis, skipna)
             return map_partitions(f, None, s, n, token=name)
 
     @derived_from(pd.DataFrame)
     def var(self, axis=None, skipna=True, ddof=1):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.var(axis=1, skipna=skipna, ddof=ddof)
-            name = '{0}var(axis=1, ddof={1})'.format(self._token_prefix, ddof)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_var, None, self,
+                                  token=self._token_prefix + 'var',
+                                  axis=axis, skipna=skipna, ddof=ddof)
         else:
             num = self._get_numeric_data()
             x = 1.0 * num.sum(skipna=skipna)
@@ -661,19 +708,20 @@ class _Frame(Base):
                     return result
                 except ZeroDivisionError:
                     return np.nan
-            name = '{0}var(ddof={1})'.format(self._token_prefix, ddof)
+            name = self._token_prefix + 'var-%s' % tokenize(self, axis, skipna, ddof)
             return map_partitions(f, None, x2, x, n, token=name)
 
     @derived_from(pd.DataFrame)
     def std(self, axis=None, skipna=True, ddof=1):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.std(axis=1, skipna=skipna, ddof=ddof)
-            name = '{0}std(axis=1, ddof={1})'.format(self._token_prefix, ddof)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_std, None, self,
+                                  token=self._token_prefix + 'std',
+                                  axis=axis, skipna=skipna, ddof=ddof)
         else:
             v = self.var(skipna=skipna, ddof=ddof)
-            name = '{0}std(ddof={1})'.format(self._token_prefix, ddof)
+            name = self._token_prefix + 'std-finish--%s' % tokenize(self, axis,
+                                                            skipna, ddof)
             return map_partitions(np.sqrt, None, v, token=name)
 
     def quantile(self, q=0.5, axis=0):
@@ -685,7 +733,7 @@ class _Frame(Base):
         q : list/array of floats, default 0.5 (50%)
             Iterable of numbers ranging from 0 to 1 for the desired quantiles
         axis : {0, 1, 'index', 'columns'} (default 0)
-            0 or 'index' for row-wise, 1 or 'columns' for column-wis
+            0 or 'index' for row-wise, 1 or 'columns' for column-wise
         """
         axis = self._validate_axis(axis)
         name = 'quantiles-concat--' + tokenize(self, q, axis)
@@ -945,7 +993,7 @@ class Series(_Frame):
     def dtype(self):
         """ Return data type """
         if self._known_dtype:
-            return self.dtype
+            return self._pd.dtype
         else:
             self._pd, self._known_dtype = self._build_pd(self.head())
             return self._pd.dtype
@@ -993,6 +1041,12 @@ class Series(_Frame):
         """
         return quantile(self, q)
 
+    def _repartition_quantiles(self, npartitions, upsample=1.0):
+        """ Approximate quantiles of Series used for repartitioning
+        """
+        from .partitionquantiles import partition_quantiles
+        return partition_quantiles(self, npartitions, upsample=upsample)
+
     @derived_from(pd.Series)
     def resample(self, rule, how=None, closed=None, label=None):
         from .tseries.resample import _resample
@@ -1000,7 +1054,7 @@ class Series(_Frame):
 
     def __getitem__(self, key):
         if isinstance(key, Series) and self.divisions == key.divisions:
-            name = 'series-index-%s[%s]' % (self._name, key._name)
+            name = 'index-%s' % tokenize(self, key)
             dsk = dict(((name, i), (operator.getitem, (self._name, i),
                                                        (key._name, i)))
                         for i in range(self.npartitions))
@@ -1026,14 +1080,15 @@ class Series(_Frame):
         # convert to numeric axis
         return {None: 0, 'index': 0}.get(axis, axis)
 
-    def _aca_agg(self, token, func, aggfunc=None):
+    def _aca_agg(self, token, func, aggfunc=None, **kwargs):
         """ Wrapper for aggregations """
         if aggfunc is None:
             aggfunc = func
 
         return aca([self], chunk=func,
-                   aggregate=lambda x: aggfunc(pd.Series(x)),
-                   columns=return_scalar, token=self._token_prefix + token)
+                   aggregate=lambda x, **kwargs: aggfunc(pd.Series(x), **kwargs),
+                   columns=return_scalar, token=self._token_prefix + token,
+                   **kwargs)
 
     @derived_from(pd.Series)
     def groupby(self, index, **kwargs):
@@ -1093,9 +1148,10 @@ class Series(_Frame):
         uniques : Series
         """
         # unique returns np.ndarray, it must be wrapped
-        chunk = lambda x: pd.Series(pd.Series.unique(x), name=self.name)
+        name = self.name
+        chunk = lambda x: pd.Series(pd.Series.unique(x), name=name)
         return aca(self, chunk=chunk, aggregate=chunk,
-                   columns=self.name, token='unique')
+                   columns=name, token='unique')
 
     @derived_from(pd.Series)
     def nunique(self):
@@ -1104,10 +1160,7 @@ class Series(_Frame):
     @derived_from(pd.Series)
     def value_counts(self):
         chunk = lambda s: s.value_counts()
-        if LooseVersion(pd.__version__) > '0.16.2':
-            agg = lambda s: s.groupby(level=0).sum().sort_values(ascending=False)
-        else:
-            agg = lambda s: s.groupby(level=0).sum().sort(inplace=False, ascending=False)
+        agg = lambda s: s.groupby(level=0).sum().sort_values(ascending=False)
         return aca(self, chunk=chunk, aggregate=agg, columns=self.name,
                    token='value-counts')
 
@@ -1132,7 +1185,8 @@ class Series(_Frame):
 
     @derived_from(pd.Series)
     def astype(self, dtype):
-        return map_partitions(pd.Series.astype, self.name, self, dtype=dtype)
+        return map_partitions(pd.Series.astype, self._pd.astype(dtype),
+                              self, dtype=dtype)
 
     @derived_from(pd.Series)
     def dropna(self):
@@ -1202,7 +1256,7 @@ class Series(_Frame):
             If list is given, the result is a DataFrame which columns is
             specified list. Otherwise, the result is a Series which name is
             given scalar or None (no name). If name keyword is not given, dask
-            tries to infer the result type using its beggining of data. This
+            tries to infer the result type using its beginning of data. This
             inference may take some time and lead to unexpected result.
         args: tuple
             Positional arguments to pass to function in addition to the array/series
@@ -1284,11 +1338,11 @@ class Index(Series):
     @derived_from(pd.Index)
     def max(self):
         # it doesn't support axis and skipna kwds
-        return self._aca_agg(token='max', func=lambda x: x.max())
+        return self._aca_agg(token='max', func=_max)
 
     @derived_from(pd.Index)
     def min(self):
-        return self._aca_agg(token='min', func=lambda x: x.min())
+        return self._aca_agg(token='min', func=_min)
 
     def count(self):
         f = lambda x: pd.notnull(x).sum()
@@ -1337,6 +1391,11 @@ class DataFrame(_Frame):
     def _constructor(self):
         return DataFrame
 
+    @property
+    def _pd_nonempty(self):
+        """ A Pandas dataframe with the same metadata but fake data."""
+        return nonempty_sample_df(self._pd)
+
     @property
     def columns(self):
         return self._pd.columns
@@ -1351,22 +1410,26 @@ class DataFrame(_Frame):
         self._name = renamed._name
 
     def __getitem__(self, key):
-
+        name = 'getitem-%s' % tokenize(self, key)
         if np.isscalar(key):
+
+            if isinstance(self._pd.index, (pd.DatetimeIndex, pd.PeriodIndex)):
+                if key not in self._pd.columns:
+                    return self._loc(key)
+
             # error is raised from pandas
             dummy = self._pd[_extract_pd(key)]
-
-            name = '{0}.{1}'.format(self._name, key)
             dsk = dict(((name, i), (operator.getitem, (self._name, i), key))
-                        for i in range(self.npartitions))
+                       for i in range(self.npartitions))
             return self._constructor_sliced(merge(self.dask, dsk), name,
                                             dummy, self.divisions)
+        elif isinstance(key, slice):
+            return self._loc(key)
 
         if isinstance(key, list):
             # error is raised from pandas
             dummy = self._pd[_extract_pd(key)]
 
-            name = '%s[%s]' % (self._name, str(key))
             dsk = dict(((name, i), (operator.getitem,
                                     (self._name, i), (list, key)))
                         for i in range(self.npartitions))
@@ -1378,7 +1441,6 @@ class DataFrame(_Frame):
             if self.divisions != key.divisions:
                 from .multi import _maybe_align_partitions
                 self, key = _maybe_align_partitions([self, key])
-            name = 'series-slice-%s[%s]' % (self._name, key._name)
             dsk = dict(((name, i), (self._partition_type._getitem_array,
                                      (self._name, i),
                                      (key._name, i)))
@@ -1387,6 +1449,18 @@ class DataFrame(_Frame):
                                      self, self.divisions)
         raise NotImplementedError(key)
 
+    def __setitem__(self, key, value):
+        if isinstance(key, (tuple, list)):
+            df = self.assign(**{k: value[c]
+                                for k, c in zip(key, value.columns)})
+        else:
+            df = self.assign(**{key: value})
+
+        self.dask = df.dask
+        self._name = df._name
+        self._pd = df._pd
+        self._known_dtype = df._known_dtype
+
     def __getattr__(self, key):
         try:
             return self[key]
@@ -1395,7 +1469,7 @@ class DataFrame(_Frame):
 
     def __dir__(self):
         return sorted(set(dir(type(self)) + list(self.__dict__) +
-                      list(self.columns)))
+                      list(filter(pd.compat.isidentifier, self.columns))))
 
     @property
     def ndim(self):
@@ -1411,8 +1485,16 @@ class DataFrame(_Frame):
             self._pd, self._known_dtype = self._build_pd(self.head())
             return self._pd.dtypes
 
+    @derived_from(pd.DataFrame)
+    def notnull(self):
+        return self.map_partitions(pd.DataFrame.notnull)
+
+    @derived_from(pd.DataFrame)
+    def isnull(self):
+        return self.map_partitions(pd.DataFrame.isnull)
+
     def set_index(self, other, drop=True, sorted=False, **kwargs):
-        """ Set the DataFrame index 9row labels) using an existing column
+        """ Set the DataFrame index (row labels) using an existing column
 
         This operation in dask.dataframe is expensive.  If the input column is
         sorted then we accomplish the set_index in a single full read of that
@@ -1474,6 +1556,28 @@ class DataFrame(_Frame):
         return DataFrameGroupBy(self, key, **kwargs)
 
     def categorize(self, columns=None, **kwargs):
+        """
+        Convert columns of the DataFrame to catefory dtype
+
+        Parameters
+        ----------
+        columns : list, optional
+            A list of column names to convert to the category type. By
+            default any column with an object dtype is converted to a
+            categorical.
+        kwargs
+            Keyword arguments are passed on to compute.
+
+        Notes
+        -----
+        When dealing with columns of repeated text values converting to
+        categorical type is often much more performant, both in terms of memory
+        and in writing to disk or communication over the network.
+
+        See also
+        --------
+        dask.dataframes.categorical.categorize
+        """
         from dask.dataframe.categorical import categorize
         return categorize(self, columns, **kwargs)
 
@@ -1508,7 +1612,7 @@ class DataFrame(_Frame):
         The original docstring follows below:\n
         """ + pd.DataFrame.query.__doc__
 
-        name = '%s.query(%s)' % (self._name, expr)
+        name = 'query-%s' % tokenize(self, expr)
         if kwargs:
             name = name + '--' + tokenize(kwargs)
             dsk = dict(((name, i), (apply, pd.DataFrame.query,
@@ -1522,6 +1626,14 @@ class DataFrame(_Frame):
         return self._constructor(merge(dsk, self.dask), name,
                                  dummy, self.divisions)
 
+    @derived_from(pd.DataFrame)
+    def eval(self, expr, inplace=None, **kwargs):
+        if '=' in expr and inplace in (True, None):
+            raise NotImplementedError("Inplace eval not supported."
+            " Please use inplace=False")
+        meta = self._pd.eval(expr, inplace=inplace, **kwargs)
+        return self.map_partitions(_eval, meta, expr, inplace=inplace, **kwargs)
+
     @derived_from(pd.DataFrame)
     def dropna(self, how='any', subset=None):
         # for cloudpickle
@@ -1577,16 +1689,19 @@ class DataFrame(_Frame):
         # convert to numeric axis
         return {None: 0, 'index': 0, 'columns': 1}.get(axis, axis)
 
-    def _aca_agg(self, token, func, aggfunc=None):
+    def _aca_agg(self, token, func, aggfunc=None, **kwargs):
         """ Wrapper for aggregations """
         if aggfunc is None:
             aggfunc = func
 
+        def aggregate(x, **kwargs):
+            return x.groupby(level=0).apply(aggfunc, **kwargs)
+
         # groupby.aggregation doesn't support skipna,
         # using gropuby.apply(aggfunc) is a workaround to handle each group as df
-        return aca([self], chunk=func,
-                   aggregate=lambda x: x.groupby(level=0).apply(aggfunc),
-                   columns=None, token=self._token_prefix + token)
+        return aca([self], chunk=func, aggregate=aggregate,
+                   columns=None, token=self._token_prefix + token,
+                   **kwargs)
 
     @derived_from(pd.DataFrame)
     def drop(self, labels, axis=0):
@@ -1691,7 +1806,7 @@ class DataFrame(_Frame):
             If list is given, the result is a DataFrame which columns is
             specified list. Otherwise, the result is a Series which name is
             given scalar or None (no name). If name keyword is not given, dask
-            tries to infer the result type using its beggining of data. This
+            tries to infer the result type using its beginning of data. This
             inference may take some time and lead to unexpected result
         args : tuple
             Positional arguments to pass to function in addition to the array/series
@@ -1740,6 +1855,22 @@ class DataFrame(_Frame):
         empty = self._pd.astype(dtype)
         return map_partitions(pd.DataFrame.astype, empty, self, dtype=dtype)
 
+    def info(self):
+        """
+        Concise summary of a Dask DataFrame.
+        """
+        lines = list()
+        lines.append(str(type(self)))
+        lines.append('Data columns (total %d columns):' % len(self.columns))
+        dtypes = self.dtypes
+        space = max([len(k) for k in self.columns]) + 4
+        template = "%s%s"
+        for i, col in enumerate(self.columns):
+            dtype = dtypes.iloc[i]
+            lines.append(template % (('%s' % col)[:space].ljust(space), dtype))
+
+        print('\n'.join(lines))
+
 
 # bind operators
 for op in [operator.abs, operator.add, operator.and_, operator_div,
@@ -1788,72 +1919,11 @@ def _assign(df, *pairs):
     return df.assign(**kwargs)
 
 
-def _partition_of_index_value(divisions, val):
-    """ In which partition does this value lie?
-
-    >>> _partition_of_index_value([0, 5, 10], 3)
-    0
-    >>> _partition_of_index_value([0, 5, 10], 8)
-    1
-    >>> _partition_of_index_value([0, 5, 10], 100)
-    1
-    >>> _partition_of_index_value([0, 5, 10], 5)  # left-inclusive divisions
-    1
-    """
-    if divisions[0] is None:
-        raise ValueError(
-            "Can not use loc on DataFrame without known divisions")
-    val = _coerce_loc_index(divisions, val)
-    i = bisect.bisect_right(divisions, val)
-    return min(len(divisions) - 2, max(0, i - 1))
-
-
-def _loc(df, start, stop, include_right_boundary=True):
-    """
-
-    >>> df = pd.DataFrame({'x': [10, 20, 30, 40, 50]}, index=[1, 2, 2, 3, 4])
-    >>> _loc(df, 2, None)
-        x
-    2  20
-    2  30
-    3  40
-    4  50
-    >>> _loc(df, 1, 3)
-        x
-    1  10
-    2  20
-    2  30
-    3  40
-    >>> _loc(df, 1, 3, include_right_boundary=False)
-        x
-    1  10
-    2  20
-    2  30
-    """
-    result = df.loc[start:stop]
-    if not include_right_boundary:
-        right_index = result.index.get_slice_bound(stop, 'left', 'loc')
-        result = result.iloc[:right_index]
-    return result
-
-
-def _coerce_loc_index(divisions, o):
-    """ Transform values to be comparable against divisions
-
-    This is particularly valuable to use with pandas datetimes
-    """
-    if divisions and isinstance(divisions[0], datetime):
-        return pd.Timestamp(o)
-    if divisions and isinstance(divisions[0], np.datetime64):
-        return np.datetime64(o).astype(divisions[0].dtype)
-    return o
-
-
 def elemwise(op, *args, **kwargs):
     """ Elementwise operation for dask.Dataframes """
     columns = kwargs.pop('columns', no_default)
 
-    _name = 'elemwise-' + tokenize(op, kwargs, *args)
+    _name = funcname(op) + '-' + tokenize(op, kwargs, *args)
 
     args = _maybe_from_pandas(args)
 
@@ -1867,7 +1937,6 @@ def elemwise(op, *args, **kwargs):
     other = [(i, arg) for i, arg in enumerate(args)
              if not isinstance(arg, (_Frame, Scalar))]
 
-
     # adjust the key length of Scalar
     keys = [d._keys() *  n if isinstance(d, Scalar)
             else d._keys() for d in dasks]
@@ -1937,11 +2006,11 @@ def reduction(x, chunk, aggregate, token=None):
     token = token or 'reduction'
     a = '{0}--chunk-{1}'.format(token, token_key)
     dsk = dict(((a, i), (empty_safe, chunk, (x._name, i)))
-                for i in range(x.npartitions))
+               for i in range(x.npartitions))
 
     b = '{0}--aggregation-{1}'.format(token, token_key)
     dsk2 = {(b, 0): (aggregate, (remove_empties,
-                     [(a,i) for i in range(x.npartitions)]))}
+                     [(a, i) for i in range(x.npartitions)]))}
 
     return Scalar(merge(x.dask, dsk, dsk2), b)
 
@@ -1954,7 +2023,8 @@ def _maybe_from_pandas(dfs):
 
 
 def apply_concat_apply(args, chunk=None, aggregate=None,
-                       columns=no_default, token=None):
+                       columns=no_default, token=None, chunk_kwargs=None,
+                       aggregate_kwargs=None, **kwargs):
     """ Apply a function to blocks, the concat, then apply again
 
     Parameters
@@ -1976,6 +2046,13 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
 
     >>> apply_concat_apply([a, b], chunk=chunk, aggregate=agg)  # doctest: +SKIP
     """
+    if chunk_kwargs is None:
+        chunk_kwargs = dict()
+    if aggregate_kwargs is None:
+        aggregate_kwargs = dict()
+    chunk_kwargs.update(kwargs)
+    aggregate_kwargs.update(kwargs)
+
     if not isinstance(args, (tuple, list)):
         args = [args]
 
@@ -1986,19 +2063,22 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
     token = token or 'apply-concat-apply'
 
     a = '{0}--first-{1}'.format(token, token_key)
-    if len(args) == 1 and isinstance(args[0], _Frame):
+    if len(args) == 1 and isinstance(args[0], _Frame) and not chunk_kwargs:
         dsk = dict(((a, i), (chunk, key))
                    for i, key in enumerate(args[0]._keys()))
     else:
         dsk = dict(((a, i), (apply, chunk, [(x._name, i)
                                             if isinstance(x, _Frame)
-                                            else x for x in args]))
+                                            else x for x in args],
+                                    kwargs))
                for i in range(args[0].npartitions))
 
     b = '{0}--second-{1}'.format(token, token_key)
-    dsk2 = {(b, 0): (aggregate,
-                      (_concat,
-                        (list, [(a, i) for i in range(args[0].npartitions)])))}
+    conc = (_concat, (list, [(a, i) for i in range(args[0].npartitions)]))
+    if not aggregate_kwargs:
+        dsk2 = {(b, 0): (aggregate, conc)}
+    else:
+        dsk2 = {(b, 0): (apply, aggregate, [conc], aggregate_kwargs)}
 
     if columns is no_default:
         return_type = type(args[0])
@@ -2049,7 +2129,7 @@ def _get_return_type(arg, metadata):
 
 def _extract_pd(x):
     """
-    Extract intenal cache data (``_pd``) from dd.DataFrame / dd.Series
+    Extract internal cache data (``_pd``) from dd.DataFrame / dd.Series
     """
     if isinstance(x, _Frame):
         return x._pd
@@ -2089,8 +2169,8 @@ def map_partitions(func, metadata, *args, **kwargs):
 
     assert callable(func)
     token = kwargs.pop('token', None)
-    token_key = tokenize(token or func, metadata, kwargs, *args)
-    name = '{0}-{1}'.format(token or 'map-partitions', token_key)
+    name = token or funcname(func)
+    name = '{0}-{1}'.format(name, tokenize(metadata, *args, **kwargs))
 
     if all(isinstance(arg, Scalar) for arg in args):
         dask = {(name, 0):
@@ -2215,12 +2295,14 @@ def quantile(df, q):
     # currently, only Series has quantile method
     if isinstance(q, (list, tuple, np.ndarray)):
         # Index.quantile(list-like) must be pd.Series, not pd.Index
-        merge_type = lambda v: pd.Series(v, index=q, name=df.name)
+        df_name = df.name
+        merge_type = lambda v: pd.Series(v, index=q, name=df_name)
         return_type = df._constructor
         if issubclass(return_type, Index):
             return_type = Series
     else:
-        merge_type = lambda v: df._partition_type(v).item()
+        typ = df._partition_type
+        merge_type = lambda v: typ(v).item()
         return_type = df._constructor_sliced
         q = [q]
 
@@ -2373,7 +2455,7 @@ def pd_split(df, p, random_state=None):
 def _take_last(a, skipna=True):
     """
     take last row (Series) of DataFrame / last value of Seriese
-    considerning NaN.
+    considering NaN.
 
     Parameters
     ----------
@@ -2531,7 +2613,7 @@ def repartition_divisions(a, b, name, out1, out2, force=False):
             tmp.append((out1, i))
             i += 1
         if len(tmp) == 0:
-            # dumy slice to return empty DataFrame or Series,
+            # dummy slice to return empty DataFrame or Series,
             # which retain original data attributes (columns / name)
             d[(out2, j - 1)] = (_loc, (name, 0), a[0], a[0], False)
         elif len(tmp) == 1:
@@ -2648,7 +2730,6 @@ class DatetimeAccessor(Accessor):
         return getattr(obj.dt, attr)(*args)
 
 
-
 class StringAccessor(Accessor):
     """ Accessor object for string properties of the Series values.
 
@@ -2668,13 +2749,6 @@ class StringAccessor(Accessor):
         return getattr(obj.str, attr)(*args)
 
 
-def try_loc(df, ind):
-    try:
-        return df.loc[ind]
-    except KeyError:
-        return df.head(0)
-
-
 def set_sorted_index(df, index, drop=True, **kwargs):
     if not isinstance(index, Series):
         index2 = df[index]
@@ -2702,3 +2776,35 @@ def set_sorted_index(df, index, drop=True, **kwargs):
 
 def _set_sorted_index(df, idx, drop):
     return df.set_index(idx, drop=drop)
+
+
+def _eval(df, expr, **kwargs):
+    return df.eval(expr, **kwargs)
+
+
+def _sum(x, **kwargs):
+    return x.sum(**kwargs)
+
+
+def _min(x, **kwargs):
+    return x.min(**kwargs)
+
+
+def _max(x, **kwargs):
+    return x.max(**kwargs)
+
+
+def _count(x, **kwargs):
+    return x.count(**kwargs)
+
+
+def _mean(x, **kwargs):
+    return x.mean(**kwargs)
+
+
+def _var(x, **kwargs):
+    return x.var(**kwargs)
+
+
+def _std(x, **kwargs):
+    return x.std(**kwargs)
